#ifndef _RIPV2_ROUTE_TABLE_H
#define _RIPV2_ROUTE_TABLE_H

#include "../ipv4_base/ipv4.h"
#include "../ipv4_route_table/ipv4_route_table.h"

#include <timerms.h>
#include <time.h>

#define RIP_MAX_TIMEOUT 180*1000 /* 180 segundos de timeout, modificable para pruebas */
#define RIP_INF_TIMEOUT -1
#define RIP_GARBAGE_TIMER 120*1000 /* 120 segundos de garbage, modificable para pruebas */


#include <stdio.h>

typedef struct ripv2_route {
 ipv4_addr_t subnet_addr;
 ipv4_addr_t subnet_mask;
 char iface[IFACE_NAME_MAX_LENGTH];
 ipv4_addr_t next_hop; /* NOTA! El siguiente salto soy yo (0.0.0.0) siempre que anuncio rutas aprendidas de routers RIP,
							 en caso de que el siguiente salto no hable RIP será él el siguiente salto. */
 uint32_t metric;
 timerms_t timer; /* Tiempo para que la ruta expire, por defecto empezamos en 180000. Es un timeout, el timer va por separado... */
} ripv2_route_t;

ripv2_route_t * ripv2_route_create ( ipv4_addr_t subnet, ipv4_addr_t mask, char* iface, ipv4_addr_t next_hop, uint32_t metric, long int timeout );

int ripv2_route_lookup ( ripv2_route_t * route, ipv4_addr_t addr );

void ripv2_route_print ( ripv2_route_t * route );

void ripv2_route_free ( ripv2_route_t * route );


/* Número de entradas máximo de la tabla de rutas RIPv2 */
#define RIPv2_ROUTE_TABLE_SIZE 256

typedef struct ripv2_route_table ripv2_route_table_t;

ripv2_route_table_t * ripv2_route_table_create();

int ripv2_route_table_add ( ripv2_route_table_t * table, ripv2_route_t * route );

ripv2_route_t * ripv2_route_table_remove ( ripv2_route_table_t * table, int index );

ripv2_route_t * ripv2_route_table_lookup ( ripv2_route_table_t * table, ipv4_addr_t addr );

ripv2_route_t * ripv2_route_table_get ( ripv2_route_table_t * table, int index );

int ripv2_route_table_find ( ripv2_route_table_t * table, ipv4_addr_t subnet, ipv4_addr_t mask );

void ripv2_route_table_free ( ripv2_route_table_t * table );

int ripv2_route_table_read ( char * filename, ripv2_route_table_t * table );

int ripv2_route_table_write ( ripv2_route_table_t * table, char * filename );

void ripv2_route_table_print ( ripv2_route_table_t * table );

#endif /* _RIPV2_ROUTE_TABLE_H */
