#include "../../files/ipv4_base/ipv4.h"
#include "../../files/eth_base/eth.h"
#include <stdio.h>
#include <libgen.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define CONFIG_FILE "./files/ipv4_config/ipv4_config_client.txt"
#define ROUTE_TABLE_FILE "./files/ipv4_route_table/ipv4_route_table_client.txt"
#define MY_PROTO_IP 150

/* int main ( int argc, char * argv[] );
 *
 * DESCRIPCIÓN:
 *   Función principal del cliente IPv4. Lee los argumentos escritos
 *   en línea de comandos, envía datos a un servidor IPv4 y recibe
 *   su respuesta.
 *
 * PARÁMETROS:
 *   'argc': Número de parámetros pasados al programa por la línea de
 *           comandos, incluyendo el propio nombre del programa.
 *   argv[1]: Dirección IPv4 destino

 * VALOR DEVUELTO:
 *   Devuelve -1 si hay errores
 */
 int main ( int argc, char * argv[] ) {

  char * myself = basename(argv[0]);

  if (argc != 3){
    printf("Uso: %s <IP> <size>\n", myself);
    printf("        <IP>: Dirección destino del servidor IP\n");
    printf("        <size>: Tamaño del payload del paquete\n");

    return -1;
  }

  /* Tratamiento de argumentos */
  char* ipv4_str = argv[1];
  ipv4_addr_t ip_dst;
  int err_ip =  ipv4_str_addr(ipv4_str, ip_dst);

  if (err_ip == -1){
    return -1;
  }

  char * payload_length_str = argv[2];
  char* endptr;
  int payload_length = (int) strtol(payload_length_str, &endptr, 0);

  printf("%d\n", payload_length);

  if(payload_length == 0 || payload_length > IPv4_MTU) {
    payload_length = IPv4_MTU;
  }

  /* Abre fichero tabla de rutas*/

  ipv4_layer_t * ipv4_layer = ipv4_open(CONFIG_FILE, ROUTE_TABLE_FILE);
  int timeout = 2000; /* 2000 milisegundos de timeout */

  unsigned char payload[payload_length]; /* Datos a enviar */
  int i; /* Vamos a rellenar los datos con números consecutivos */
  for (i=0; i<payload_length; i++) {
    payload[i] = (unsigned char)i;
  }

  int err_send = ipv4_send(ipv4_layer, ip_dst, (uint8_t) MY_PROTO_IP, payload, payload_length);

  if(err_send == -1){
    return -1;
  }
  if(err_send == 0){
    exit(0);
  }

  printf("%d bytes enviados.\n", err_send);

  unsigned char buffer_rcvd[IPv4_MTU];
  ipv4_addr_t ip_src;

  int err_recv = ipv4_receive(ipv4_layer, ip_src, (uint8_t) MY_PROTO_IP, buffer_rcvd, payload_length, timeout);

  if(err_recv==-1){
    return -1;
  }

  if(err_recv==0){
    exit(0);
  }

  char ip_src_str[IPv4_STR_MAX_LENGTH];
  ipv4_addr_str(ip_src, ip_src_str);
  printf("%d bytes recibidos de: %s\n", err_recv, ip_src_str);
  print_pkt(buffer_rcvd,err_recv,0);

  int err_close = ipv4_close(ipv4_layer);

  if(err_close){
    return -1;
  }

 return 0;
}
