/*rawnetcc /tmp/client files/eth_base/eth.c files/arp/arp.c files/ipv4_base/ipv4.c files/ipv4_config/ipv4_config.c files/ipv4_route_table/ipv4_route_table.c files/udp/udp.c files/rip/rip.c files/rip_route_table/rip_route_table.c testing/clients/rip_client.c */

#include "../../files/rip/rip.h"

#include <stdio.h>
#include <libgen.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <netinet/in.h>
#include <timerms.h>
#include <time.h>

#define CONFIG_FILE "./files/ipv4_config/ipv4_config_client.txt"
#define ROUTE_TABLE_FILE "./files/ipv4_route_table/ipv4_route_table_client.txt"

/* int main ( int argc, char * argv[] );
 *
 * DESCRIPCIÓN:
 *   Función principal del cliente UDP. Lee los argumentos escritos
 *   en línea de comandos, envía datos a un servidor UDP y recibe
 *   su respuesta.
 *
 * PARÁMETROS:
 *   'argc': Número de parámetros pasados al programa por la línea de
 *           comandos, incluyendo el propio nombre del programa.
 *   argv[1]: Dirección IPv4 destino
 *   argv[2]: Puerto destino del paquete
 *
 * VALOR DEVUELTO:
 *   Devuelve -1 si hay errores
 */
 int main ( int argc, char * argv[] ) {

  char * myself = basename(argv[0]);

  if (argc != 2){
    printf("Uso: %s <IP>\n", myself);
    printf("        <IP>: Dirección destino del servidor IP\n");

    exit(-1);
  }

  /* Tratamiento de argumentos */
  char* ipv4_str = argv[1];
  ipv4_addr_t ip_dst;
  int err_ip =  ipv4_str_addr(ipv4_str, ip_dst);

  if (err_ip == -1){
    return -1;
  }

  int payload_length = RIP_HEADER_LENGTH + RIP_ENTRY_LENGTH ;

  udp_layer_t * udp_layer = udp_open(CONFIG_FILE, ROUTE_TABLE_FILE, 0);
  int timeout = 2000; /* 2000 milisegundos de timeout */



  /* Estructura para pedir todo */
  rip_entry_t entry;
    entry.addr_family_id = 0x0000;
    entry.route_tag = 0x0000;
    memcpy(entry.subnet_mask, IPv4_ZERO_ADDR, IPv4_ADDR_SIZE);
    memcpy(entry.next_hop, IPv4_ZERO_ADDR, IPv4_ADDR_SIZE);
    entry.metric = htonl(RIP_INF_METRIC);

  rip_packet_t request;
    request.command = RIP_COMMAND_REQUEST;
    request.version = RIP_VERSION;
    request.zeros = 0x0000;
    request.entries[0] = entry;

  int err_send = udp_send(udp_layer, ip_dst, RIP_PORT_UDP, (unsigned char *) &request, payload_length);

  if(err_send == -1){
    return -1;
  }
  if(err_send == 0){
    exit(0);
  }
  printf("%d bytes enviados.\n", err_send);

  unsigned char buffer_rcvd[RIP_MAX_LENGTH];
  ipv4_addr_t ip_src;
  uint16_t port_rcvd = 0;

  int bytes_recv = udp_receive(udp_layer, ip_src, (uint16_t *) &port_rcvd, buffer_rcvd, RIP_MAX_LENGTH, timeout);

  if(bytes_recv==-1){
    printf("rip_client: error al recibir.\n");
    exit(-1);
  }
  if(bytes_recv==0){
    printf("RIP Timeout.\n");
    exit(0);
  }

  int rip_entries = rip_len2entries(bytes_recv);

  rip_packet_t * response = (struct rip_packet *)buffer_rcvd;

  rip_print_packet(response, rip_entries);

  int err_close = udp_close(udp_layer);

  if(err_close){
    return -1;
  }

 return 0;

}
