/* rawnetcc /tmp/server files/eth_base/eth.c files/arp/arp.c files/ipv4_base/ipv4.c files/ipv4_config/ipv4_config.c files/ipv4_route_table/ipv4_route_table.c files/udp/udp.c files/rip/rip.c files/rip_route_table/rip_route_table.c testing/servers/rip_server.c */

#include <stdio.h>
#include <libgen.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <netinet/in.h>
#include <stdio.h>

#include "../../files/rip/rip.h"
#include "../../files/rip_route_table/rip_route_table.h"

#define CONFIG_FILE "./files/ipv4_config/ipv4_config_server.txt"
#define IP_ROUTE_TABLE_FILE "./files/ipv4_route_table/ipv4_route_table_server.txt"
#define RIP_ROUTE_TABLE_FILE "./files/rip_route_table/rip_route_table_server.txt"


/* Tabla de rutas RIP global */
ripv2_route_table_t * table = NULL;
char * rip_iface = NULL;


/* rfc2453: 3.9.1 Request Messages:
  The Request is processed entry by entry.  If there are no entries, no
  response is given.  There is one special case.  If there is exactly
  one entry in the request, and it has an address family identifier of
  zero and a metric of infinity (i.e., 16), then this is a request to
  send the entire routing table.
*/

/*void rip_request_all_entries(udp_layer_t * udp_layer, ipv4_addr_t ip_src, uint16_t port_src) {

	// Estructura para pedir todo
	rip_entry_t entry;
		entry.addr_family_id = 0x0000;
		entry.route_tag = 0x0000;
		memcpy(entry.subnet_mask, IPv4_ZERO_ADDR, IPv4_ADDR_SIZE);
		memcpy(entry.next_hop, IPv4_ZERO_ADDR, IPv4_ADDR_SIZE);
		entry.metric = htonl(RIP_INF_METRIC);

	rip_packet_t request;
		request.command = RIP_COMMAND_REQUEST;
		request.version = RIP_VERSION;
		request.zeros = 0x0000;
		request.entries[0] = entry;

	int err_send = udp_send(udp_layer, ip_src, port_src, (unsigned char *) &request, RIP_HEADER_LENGTH + RIP_ENTRY_LENGTH); // 1 sola entry

	if (err_send == -1) {
		printf("Error en udp_send()\n");
		exit(-1);
	} else if (err_send == 0) {
		printf("No se han enviado datos.\n");
		exit(0);
	}
}
*/

void rip_send_all_entries(udp_layer_t * udp_layer, ipv4_addr_t ip_src, uint16_t port_src){

	rip_packet_t rip_response;
	int i = 0;
	int num_entries = 0;
	/* Enviamos todas las rutas, pero comprobamos que haya para enviar */

	rip_response.command = RIP_COMMAND_RESPONSE;
	rip_response.version = RIP_VERSION;
	rip_response.zeros = 0x0000;

	for (i = 0; i < RIPv2_ROUTE_TABLE_SIZE; i++) {

		ripv2_route_t * route_i = ripv2_route_table_get(table, i);

		if (route_i != NULL) {
			rip_response.entries[num_entries].addr_family_id = htons(2);
			rip_response.entries[num_entries].route_tag = htons(0);
			memcpy(rip_response.entries[num_entries].ipv4_addr, route_i->subnet_addr, IPv4_ADDR_SIZE);
			memcpy(rip_response.entries[num_entries].subnet_mask, route_i->subnet_mask, IPv4_ADDR_SIZE);
			memcpy(rip_response.entries[num_entries].next_hop, IPv4_ZERO_ADDR, IPv4_ADDR_SIZE); //zero
			rip_response.entries[num_entries].metric = htonl(route_i->metric);
			num_entries++;
		}
	}

	if (num_entries==0){
		printf("No hay entries para enviar.\n");
	}else{
		/* Finalmente calculamos el tamaño total del response */
		int rip_packet_length = RIP_HEADER_LENGTH + num_entries*RIP_ENTRY_LENGTH;

		/* Preparamos y enviamos el paquete */
		unsigned char *buffer_send = NULL;
		buffer_send = (unsigned char *)&rip_response;

		int bytes_sent = udp_send(udp_layer, ip_src, port_src, buffer_send, rip_packet_length);

		if (bytes_sent == -1) {
			printf("Error en udp_send()\n");
			exit(-1);
		} else if (bytes_sent == 0) {
			printf("No se han enviado datos.\n");
			exit(0);
		}
	}
}

void rip_handle_response(rip_packet_t * rip_response, udp_layer_t * udp_layer, ipv4_addr_t ip_src, int bytes_recv) {

	int num_req_entries = rip_len2entries(bytes_recv);
	int i = 0;

  for (i = 0; i < num_req_entries; i++) {
		rip_entry_t entry = rip_response->entries[i];
		uint32_t new_metric = ntohl(entry.metric) + 1;
		if (new_metric > 16) {
			new_metric = 16;
		}

		//Si el siguiente salto es 0.0.0.0 es el router que me lo ha enviado, sino el siguiente salto es el indicado en el entry ...
		ipv4_addr_t next_hop;
		if (memcmp(entry.next_hop, IPv4_ZERO_ADDR, IPv4_ADDR_SIZE) == 0) {
			memcpy(next_hop, ip_src, IPv4_ADDR_SIZE);
		} else {
			memcpy(next_hop, entry.next_hop, IPv4_ADDR_SIZE);
		}
		int route_index = ripv2_route_table_find(table, entry.ipv4_addr, entry.subnet_mask);

		if (route_index == -1) {
			// La ruta RIP no existe (route_index = -1), creamos la ruta en caso de no estar a infinito.
			if (new_metric < RIP_INF_METRIC) {
        ripv2_route_t * new_route = NULL;
				rip_iface = udp_getiface_name(udp_layer);
				new_route = ripv2_route_create(entry.ipv4_addr, entry.subnet_mask, rip_iface, next_hop, new_metric, RIP_MAX_TIMEOUT);
				int add_route = ripv2_route_table_add(table, new_route);

				if (add_route == -1) {
						printf("Error añadiendo ruta RIP en rip_handle_response\n");
				} else {
            printf("Tabla de rutas actualizada.\n");
				}
			} else {
					printf("La ruta nueva tiene métrica infinito, no la creamos.\n");
			}
		} else {

  		ripv2_route_t * route = ripv2_route_table_get(table, route_index);

			// La ruta ya existe, debemos actualizarla en caso de que sea mejor
    	// Actualizamos el timer en caso de ser la misma. Comprobamos que tenga la misma métrica
  		// Comprobamos si la ruta no ha cambiado y sigue siendo igual a como lo aprendió
  		if ((memcmp(route->next_hop, next_hop, IPv4_ADDR_SIZE) == 0)) {
  			// Comprobamos que el padre manda infinito o 15 (infinito para nosotros)
  			if (new_metric >= RIP_INF_METRIC) {
  				// Primera vez, se marca como Garbage. Iniciamos temporizador de basura.
  				if (route->metric < RIP_INF_METRIC) {
  					route->metric = RIP_INF_METRIC;
  					timerms_reset(&route->timer, RIP_GARBAGE_TIMER);
  				}
  			} else {
  				route->metric = new_metric; // En caso de que la métrica cambie, la actualizamos (si es igual, se queda igual)
  				timerms_reset(&route->timer, RIP_MAX_TIMEOUT);
  			}
  		} else if (new_metric < route->metric) {
        	memcpy(route->next_hop, next_hop, IPv4_ADDR_SIZE);
        	route->metric = new_metric;
					strncpy(route->iface, rip_iface, IFACE_NAME_MAX_LENGTH);
        	timerms_reset(&route->timer, RIP_MAX_TIMEOUT);
  		} else {
  	    printf("Ruta RIP no actualizada!\n");
  		}
  	}
	}
}

void rip_handle_request(rip_packet_t * rip_request, udp_layer_t * udp_layer, ipv4_addr_t ip_src, uint16_t port_rcvd, int bytes_recv){
	/* Comprobamos que el paquete tiene como mínimo una entrada (request). Falta completar... */
	if (bytes_recv < (RIP_HEADER_LENGTH + RIP_ENTRY_LENGTH)) {
		printf("Paquete RIP con tamaño menor al necesario.\n");
		exit(-1);
	}

	rip_packet_t rip_response;
	int num_entries = 0;
	int num_req_entries = rip_len2entries(bytes_recv);

	/* Comprobamos que nos piden todas las rutas con las 3 condiciones necesarias */
	if(num_req_entries == 1 && rip_request->entries[0].addr_family_id == 0x0000 && ntohl(rip_request->entries[0].metric) == RIP_INF_METRIC) {
    /* Enviar todas las rutas */
    rip_send_all_entries(udp_layer, ip_src, port_rcvd);
	} else {
		/* Buscar las rutas pedidas en nuestra tabla y generar un paquete con ellas */
		for (num_entries = 0; num_entries < num_req_entries; num_entries++) {
			rip_entry_t entry = rip_request->entries[num_entries];
			int route_index = ripv2_route_table_find(table, entry.ipv4_addr, entry.subnet_mask);
			ripv2_route_t * route_req = ripv2_route_table_get(table, route_index);
			/* Si la ruta buscada la tenemos, enviamos nuestra métrica real */
			if (route_req != NULL) {
				rip_response.entries[num_entries].addr_family_id = htons(2);
				rip_response.entries[num_entries].route_tag = htons(0);
				memcpy(rip_response.entries[num_entries].ipv4_addr, route_req->subnet_addr, IPv4_ADDR_SIZE);
				memcpy(rip_response.entries[num_entries].subnet_mask, route_req->subnet_mask, IPv4_ADDR_SIZE);
				memcpy(rip_response.entries[num_entries].next_hop, IPv4_ZERO_ADDR, IPv4_ADDR_SIZE);
				rip_response.entries[num_entries].metric = htonl(route_req->metric);

			} else { /* Si la ruta buscada no la tenemos, la devolvemos con métrica a infinito */
				rip_response.entries[num_entries].addr_family_id = entry.addr_family_id;
				rip_response.entries[num_entries].route_tag = entry.route_tag;
				memcpy(rip_response.entries[num_entries].ipv4_addr, entry.ipv4_addr, IPv4_ADDR_SIZE);
				memcpy(rip_response.entries[num_entries].subnet_mask, entry.subnet_mask, IPv4_ADDR_SIZE);
				memcpy(rip_response.entries[num_entries].next_hop, route_req->next_hop, IPv4_ADDR_SIZE);
				rip_response.entries[num_entries].metric = htonl(RIP_INF_METRIC);
			}
		}

		/* Completamos el paquete de respuesta */
		rip_response.command = RIP_COMMAND_RESPONSE;
		rip_response.version = RIP_VERSION;
		rip_response.zeros = 0x0000;

		/* Finalmente calculamos el tamaño total del response */
		int rip_packet_length = rip_entries2len(num_entries);

		/* Preparamos y enviamos el paquete */
		unsigned char * payload = NULL;
		payload = (unsigned char *)&rip_response;
		int bytes_sent = udp_send(udp_layer, ip_src, port_rcvd, payload, rip_packet_length);

		if (bytes_sent == -1) {
			printf("Error en udp_send()\n");
			exit(-1);
		} else if (bytes_sent == 0) {
			printf("No se han enviado datos.\n");
			exit(0);
		}
    printf("%d bytes enviados.\n", bytes_sent);
	}
}

long int min_time ( long int a, long int b ){
	long int min;

	if (a < 0) {
		min = b;
	} else if (b < 0) {
		min = a;
	} else if (a < b) {
		min = a;
	} else {
		min = b;
	}

	return min;
}

int main ( int argc, char * argv[] ) {

  char * myself = basename(argv[0]);

  if (argc!= 1){
    printf("Uso: %s <port>\n", myself);
    exit(-1);
  }

  printf("Escuchando UDP en el puerto %d...\n", RIP_PORT_UDP);

  /* Crear tabla de rutas */
	table = ripv2_route_table_create();
	ripv2_route_table_read(RIP_ROUTE_TABLE_FILE, table);

  /* Después de leer imprimimos las rutas obtenidas desde el fichero */
	printf("Tabla de rutas RIP inicializada desde fichero.");
	ripv2_route_table_print(table);

  /* Iniciando servidor RIP sobre UDP */
  udp_layer_t * udp_layer = udp_open(CONFIG_FILE, IP_ROUTE_TABLE_FILE, RIP_PORT_UDP);
	long int min_timer;

  unsigned char buffer_rcvd[RIP_MAX_LENGTH];
  ipv4_addr_t ip_src;
  uint16_t port_rcvd = 0;
  rip_packet_t *rip_packet_recv = NULL;

	//rip_request_all_entries(udp_layer, RIP_IP_MULTICAST_ADDR, RIP_PORT_UDP);

  while(1) {
		/* Gestionamos los timers antes de hacer nada para quedarnos con el mínimo */
 		min_timer = -1;
 		int i;
 		for (i = 0; i < RIPv2_ROUTE_TABLE_SIZE; i++) {
			ripv2_route_t * route_i = ripv2_route_table_get(table, i);
			if (route_i != NULL) {
				long int time_left_i = timerms_left(&route_i->timer);
				min_timer = min_time(time_left_i, min_timer);
			}
	  }

    int bytes_recv = udp_receive(udp_layer, ip_src, &port_rcvd, buffer_rcvd, RIP_MAX_LENGTH, min_timer);

    if (bytes_recv == -1) {
      printf("rip_server: error al recibir.\n");
      exit(-1);
    } else if (bytes_recv == 0) {
      printf("Timeout.\n");

			for (i = 0; i < RIPv2_ROUTE_TABLE_SIZE; i++) {
				ripv2_route_t * route_i = ripv2_route_table_get(table, i);
				if (route_i != NULL) {
					long int time_left_i = timerms_left(&route_i->timer);
					/* El temporizador ha expirado... */
					if (time_left_i == 0) {
						/* En este caso sabemos que ha expirado el garbage collector */
						if (route_i->metric == RIP_INF_METRIC) {
							ripv2_route_free(ripv2_route_table_remove(table, i)); /* Borramos la ruta porque han expirado todos los temporizadores */
						} else {
							route_i->metric = RIP_INF_METRIC;
	    					timerms_reset(&route_i->timer, RIP_GARBAGE_TIMER);
						}
					}
				}
			}
    } else {

      rip_packet_recv = (rip_packet_t *)buffer_rcvd;

			if (rip_packet_recv->command == RIP_COMMAND_REQUEST) {

				rip_handle_request(rip_packet_recv, udp_layer, ip_src, port_rcvd, bytes_recv);

			} else if (rip_packet_recv->command == RIP_COMMAND_RESPONSE) {

				rip_handle_response(rip_packet_recv, udp_layer, ip_src, bytes_recv);

      }
		}

		ripv2_route_table_print(table);
	}

	udp_close(udp_layer);

  return 0;
}
