#include "../../files/udp/udp.h"

#include <stdio.h>
#include <libgen.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

#define CONFIG_FILE "./files/ipv4_config/ipv4_config_server.txt"
#define ROUTE_TABLE_FILE "./files/ipv4_route_table/ipv4_route_table_server.txt"

/* int main ( int argc, char * argv[] );
 *
 * DESCRIPCIÓN:
 *   Función principal del servidor UDP. Lee los argumentos escritos
 *   en línea de comandos, espera datos del client y le responde con los mismos.
 *
 * PARÁMETROS:
 *   'argc': Número de parámetros pasados al programa por la línea de
 *           comandos, incluyendo el propio nombre del programa.
 *   argv[1]: Puerto de servidor a escuchar

 * VALOR DEVUELTO:
 *   Devuelve -1 si hay errores
 */
int main ( int argc, char * argv[] ) {

  char * myself = basename(argv[0]);

  if (argc!= 2){
    printf("Uso: %s <port>\n", myself);
    printf("        <port>: Puerto a escuchar\n");

    exit(-1);
  }

  /* Leer puerto a escuchar */
  char * port_str = argv[1];
  char* endptr;
  uint16_t port_listen = (uint16_t) strtol(port_str, &endptr, 0);

  if(port_listen < UDP_MIN_PORT || port_listen > UDP_MAX_PORT) {
    printf(" Puerto debe estar en el rango 49152 a 65535\n");

    exit(-1);
  }

  printf("Escuchando UDP en el puerto %d...\n", port_listen);

  udp_layer_t * udp_layer = udp_open(CONFIG_FILE, ROUTE_TABLE_FILE, port_listen);
  int timeout = -1;

  unsigned char buffer_rcvd[MTU_UDP];
  ipv4_addr_t ip_src;
  uint16_t port_rcvd = 0;

  while(1) {
    int err_recv = udp_receive(udp_layer, ip_src, &port_rcvd, buffer_rcvd, MTU_UDP, timeout);

    if(err_recv == -1){
      exit(-1);
    }

    if(err_recv == 0){
      exit(0);
    }

    char ip_src_str[IPv4_STR_MAX_LENGTH];
    ipv4_addr_str(ip_src, ip_src_str);
    printf("%d bytes recibidos de: %s\n", err_recv, ip_src_str);
    print_pkt(buffer_rcvd,err_recv,0);

    int err_send = udp_send(udp_layer, ip_src, port_rcvd, buffer_rcvd, err_recv);

    printf("%d bytes enviados.\n", err_send);

    if(err_send == -1){
      return -1;
    }
    if(err_send == 0){
      exit(0);
    }
  }
  int err_close = udp_close(udp_layer);

  if(err_close){
    return -1;
  }
}
